# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import ToolsDataFrameRequestColumnsItem
    from . import calendar, email, sheets, structured_data_extractor, tasks
    from .sheets import UpdateSheetRangeRequestValuesItemItem
    from .tasks import RunTaskRequestTaskType
_dynamic_imports: typing.Dict[str, str] = {
    "RunTaskRequestTaskType": ".tasks",
    "ToolsDataFrameRequestColumnsItem": ".types",
    "UpdateSheetRangeRequestValuesItemItem": ".sheets",
    "calendar": ".calendar",
    "email": ".email",
    "sheets": ".sheets",
    "structured_data_extractor": ".structured_data_extractor",
    "tasks": ".tasks",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "RunTaskRequestTaskType",
    "ToolsDataFrameRequestColumnsItem",
    "UpdateSheetRangeRequestValuesItemItem",
    "calendar",
    "email",
    "sheets",
    "structured_data_extractor",
    "tasks",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class AopExecuteRequestIn(UniversalBaseModel):
    """
    Request model for executing an AOP (Agent Operating Procedure).
    """

    asset_id: str = pydantic.Field()
    """
    ID of the existing AOP asset to execute
    """

    user_inputs: typing.Optional[typing.Dict[str, typing.Optional[str]]] = pydantic.Field(default=None)
    """
    Optional user inputs to append to the AOP's prompt as key-value pairs
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from typing import Any, Dict, Optional


class ApiError(Exception):
    headers: Optional[Dict[str, str]]
    status_code: Optional[int]
    body: Any

    def __init__(
        self,
        *,
        headers: Optional[Dict[str, str]] = None,
        status_code: Optional[int] = None,
        body: Any = None,
    ) -> None:
        self.headers = headers
        self.status_code = status_code
        self.body = body

    def __str__(self) -> str:
        return f"headers: {self.headers}, status_code: {self.status_code}, body: {self.body}"



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.general_agent_request import GeneralAgentRequest
from ...types.general_agent_response import GeneralAgentResponse
from .raw_client import AsyncRawGeneralClient, RawGeneralClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class GeneralClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawGeneralClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawGeneralClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawGeneralClient
        """
        return self._raw_client

    def batch(
        self, *, request: typing.Sequence[GeneralAgentRequest], request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[GeneralAgentResponse]:
        """
        Coming soon! Call the general agent with batched requests and return the results.

        Parameters
        ----------
        request : typing.Sequence[GeneralAgentRequest]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[GeneralAgentResponse]
            Successful Response

        Examples
        --------
        from athena import Athena, GeneralAgentConfig, GeneralAgentRequest, InputMessage

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.agents.general.batch(
            request=[
                GeneralAgentRequest(
                    config=GeneralAgentConfig(
                        enabled_tools=[],
                    ),
                    messages=[
                        InputMessage(
                            content="Please call the search tool for AAPL news.",
                            role="user",
                        )
                    ],
                )
            ],
        )
        """
        _response = self._raw_client.batch(request=request, request_options=request_options)
        return _response.data

    def invoke(
        self, *, request: GeneralAgentRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> GeneralAgentResponse:
        """
        Call the general Athena agent synchronously.

        Call the agent with the messages list, wait for the agent to complete,
        and return the result.

        Parameters
        ----------
        request : GeneralAgentRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GeneralAgentResponse
            Successful Response

        Examples
        --------
        from athena import Athena, GeneralAgentConfig, GeneralAgentRequest, InputMessage

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.agents.general.invoke(
            request=GeneralAgentRequest(
                config=GeneralAgentConfig(
                    enabled_tools=[],
                ),
                messages=[
                    InputMessage(
                        content="Please call the search tool for AAPL news.",
                        role="user",
                    )
                ],
            ),
        )
        """
        _response = self._raw_client.invoke(request=request, request_options=request_options)
        return _response.data

    def stream_events(
        self, *, request: GeneralAgentRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> GeneralAgentResponse:
        """
        Coming soon! Call the general agent and stream events for real-time chat applications.

        Parameters
        ----------
        request : GeneralAgentRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GeneralAgentResponse
            Successful Response

        Examples
        --------
        from athena import Athena, GeneralAgentConfig, GeneralAgentRequest, InputMessage

        client = Athena(
            api_key="YOUR_API_KEY",
        )
        client.agents.general.stream_events(
            request=GeneralAgentRequest(
                config=GeneralAgentConfig(
                    enabled_tools=[],
                ),
                messages=[
                    InputMessage(
                        content="Please call the search tool for AAPL news.",
                        role="user",
                    )
                ],
            ),
        )
        """
        _response = self._raw_client.stream_events(request=request, request_options=request_options)
        return _response.data


class AsyncGeneralClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawGeneralClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawGeneralClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawGeneralClient
        """
        return self._raw_client

    async def batch(
        self, *, request: typing.Sequence[GeneralAgentRequest], request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[GeneralAgentResponse]:
        """
        Coming soon! Call the general agent with batched requests and return the results.

        Parameters
        ----------
        request : typing.Sequence[GeneralAgentRequest]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[GeneralAgentResponse]
            Successful Response

        Examples
        --------
        import asyncio

        from athena import (
            AsyncAthena,
            GeneralAgentConfig,
            GeneralAgentRequest,
            InputMessage,
        )

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.agents.general.batch(
                request=[
                    GeneralAgentRequest(
                        config=GeneralAgentConfig(
                            enabled_tools=[],
                        ),
                        messages=[
                            InputMessage(
                                content="Please call the search tool for AAPL news.",
                                role="user",
                            )
                        ],
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.batch(request=request, request_options=request_options)
        return _response.data

    async def invoke(
        self, *, request: GeneralAgentRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> GeneralAgentResponse:
        """
        Call the general Athena agent synchronously.

        Call the agent with the messages list, wait for the agent to complete,
        and return the result.

        Parameters
        ----------
        request : GeneralAgentRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GeneralAgentResponse
            Successful Response

        Examples
        --------
        import asyncio

        from athena import (
            AsyncAthena,
            GeneralAgentConfig,
            GeneralAgentRequest,
            InputMessage,
        )

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.agents.general.invoke(
                request=GeneralAgentRequest(
                    config=GeneralAgentConfig(
                        enabled_tools=[],
                    ),
                    messages=[
                        InputMessage(
                            content="Please call the search tool for AAPL news.",
                            role="user",
                        )
                    ],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.invoke(request=request, request_options=request_options)
        return _response.data

    async def stream_events(
        self, *, request: GeneralAgentRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> GeneralAgentResponse:
        """
        Coming soon! Call the general agent and stream events for real-time chat applications.

        Parameters
        ----------
        request : GeneralAgentRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GeneralAgentResponse
            Successful Response

        Examples
        --------
        import asyncio

        from athena import (
            AsyncAthena,
            GeneralAgentConfig,
            GeneralAgentRequest,
            InputMessage,
        )

        client = AsyncAthena(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.agents.general.stream_events(
                request=GeneralAgentRequest(
                    config=GeneralAgentConfig(
                        enabled_tools=[],
                    ),
                    messages=[
                        InputMessage(
                            content="Please call the search tool for AAPL news.",
                            role="user",
                        )
                    ],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.stream_events(request=request, request_options=request_options)
        return _response.data
